import { BehaviorSubject, Subject, combineLatest } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import * as i0 from '@angular/core';
import { Injectable, NgModule } from '@angular/core';

const defaultOptions = {
    tagAttribute: 'data-highlighted',
    highlightClass: 'highlight'
};
class Highlighter {
    constructor(options) {
        this._highlightText = '';
        this.options = Object.assign({}, defaultOptions);
        this.highlightSubject = new BehaviorSubject(this.highlightText);
        if (options) {
            Object.assign(this.options, options);
        }
        this.highlight();
    }
    get tagAttribute() { return this.options.tagAttribute; }
    get highlightClass() { return this.options.highlightClass; }
    get highlightText() {
        return this._highlightText;
    }
    set highlightText(value) {
        this._highlightText = value;
        this.highlightSubject.next(value);
    }
    destroy() {
        var _a;
        this.highlightText = '';
        (_a = this.mutationObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
        this.mutationObserver = undefined;
    }
    notEmpty(value) {
        return value !== null && value !== undefined;
    }
    ;
    highlight() {
        const modifyingSubject = new Subject();
        const callback = (mutationRecords) => {
            modifyingSubject.next(mutationRecords);
        };
        this.mutationObserver = new MutationObserver(callback);
        const s = combineLatest([modifyingSubject, this.highlightSubject.pipe(debounceTime(200))]);
        s.subscribe(([mutationRecords, h]) => {
            const r = mutationRecords
                .map(record => this.processMutation(record, this.highlightText))
                .filter(this.notEmpty);
            this.mutationObserver.disconnect();
            r.forEach(({ parentNode: existingNode, oldHTML: newHTML, replace: restore }) => {
                const oldHighlightNode = existingNode.previousSibling;
                const savedDisplay = oldHighlightNode === null || oldHighlightNode === void 0 ? void 0 : oldHighlightNode.getAttribute(this.tagAttribute);
                const oldHighlightNodeExists = oldHighlightNode && savedDisplay;
                const p = (s) => s === 'not-specified' ? '' : s;
                if (restore) {
                    if (oldHighlightNodeExists) {
                        console.log('removing old highlight node, and restoring original node');
                        oldHighlightNode.remove();
                        existingNode.style.display = p(savedDisplay);
                    }
                    else {
                        console.log('doing nothing');
                    }
                    return;
                }
                const highlightedNode = existingNode.cloneNode();
                highlightedNode.innerHTML = newHTML;
                let preserveDisplay = existingNode.style.display || 'not-specified';
                if (oldHighlightNodeExists) {
                    preserveDisplay = savedDisplay;
                    console.log('removing old highlight node, and replacing with new highlight node');
                    oldHighlightNode.remove();
                }
                else {
                    console.log('inserting highlight node, and hiding original node');
                }
                highlightedNode.setAttribute(this.tagAttribute, preserveDisplay);
                highlightedNode.style.display = p(preserveDisplay);
                existingNode.parentNode.insertBefore(highlightedNode, existingNode);
                existingNode.style.display = 'none';
            });
            this.mutationObserver.observe(document, {
                characterData: true,
                subtree: true
            });
        });
        this.mutationObserver.observe(document, { characterData: true, subtree: true });
    }
    processMutation(m, h) {
        const escaped = h.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
        const r = new RegExp(`(${escaped})+`, 'g');
        if (m.target.nodeType !== 3 ||
            !m.target.parentNode ||
            (m.target instanceof HTMLElement &&
                m.target.getAttribute(this.tagAttribute))) {
            // If this is not a text node with a valid parent, skip it
            return;
        }
        const oldHTML = m.target.parentNode.innerHTML;
        const newHTML = !h ? oldHTML : oldHTML.replaceAll(r, `<span class="${this.highlightClass}">$&</span>`);
        return { parentNode: m.target.parentNode, oldHTML: newHTML, replace: oldHTML === newHTML };
    }
}

class HighlighterService {
    constructor() { }
    createHighlighter(options) {
        return new Highlighter(options);
    }
    destroyHighlighter(h) {
        h.destroy();
    }
}
HighlighterService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: HighlighterService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
HighlighterService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: HighlighterService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: HighlighterService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

class NgxHtmlHighlighterModule {
    static forRoot() {
        const rc = {
            ngModule: NgxHtmlHighlighterModule,
            providers: [HighlighterService],
        };
        return rc;
    }
}
NgxHtmlHighlighterModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: NgxHtmlHighlighterModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxHtmlHighlighterModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: NgxHtmlHighlighterModule });
NgxHtmlHighlighterModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: NgxHtmlHighlighterModule, imports: [[]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: NgxHtmlHighlighterModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [],
                    exports: []
                }]
        }] });

/*
 * Public API Surface of ngx-html-highlighter
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Highlighter, HighlighterService, NgxHtmlHighlighterModule };
//# sourceMappingURL=ngx-html-highlighter.js.map
